;**
;*
;*  Module:      da_cfg.asm
;*  Description: console-access-method configuration for DA.EXE
;*                 search for string 'meth' in DA.EXE
;*                 append appropriate method descriptor (a, b, or m)
;*                 append negative of descriptor
;*                 write new copy to DA.TMP
;*                 delete DA.EXE
;*                 rename DA.TMP to DA.EXE
;*  Author:      Paul Roub
;*
;*  Revision History:
;*     4-11-88 : created
;*     4-12-88 : comments added.  i may have to modify this someday.
;*
;*  Notes:
;*    i would have used the nifty new simplified segment directives,  except
;*    that these don't work for .COM files.  not that .COM files are better,
;*    i'm just used to writing assembly programs in .COM format.
;*
;*  to create DA_CFG.COM:
;*    masm da_cfg;
;*    link da_cfg;
;*    exe2bin da_cfg.exe da_cfg.com
;*
;**

;**;
bufsize         equ     32 * 1024

cseg    segment
                assume  cs:cseg, es:cseg, ds:cseg, ss:cseg

                org     80h
ParamLen        label   byte

                org     100h
start:          jmp     da_cfg

val             db      'm'                      ; access-method code
eth             db      'eth'                    ; for comparison purposes
fil             db      'da.exe', 0              ; program file name
tmp             db      'da.tmp', 0              ; temporary file name
handle          dw      ?                        ; handle for DA.EXE
t_handle        dw      ?                        ; handle for DA.TMP
left            dw      ?                        ; size (in bytes) of DA.EXE

crlf            db      13, 10, '$'              ; used by err_exit

;*
;*  error messages
;*
usage           db      "usage: da_cfg <m, a, b>$"
no_fil          db      "error: can't open da.exe$"
no_tmp          db      "error: can't open temporary file da.tmp$"
no_read         db      "error reading da.exe$"
no_write        db      "error writing da.tmp$"
no_close_fil    db      "error closing da.exe$"
no_close_tmp    db      "error closing da.tmp$"
no_rename       db      "error renaming da.tmp to da.exe$"
no_meth         db      "invalid da.exe (is it version 2.0 or higher ?)$"
no_mem          db      "insufficient memory$"


;**;
;**
;*
;*  da_cfg
;*
;*  main (well, only) routine
;*
;**
da_cfg  proc near
                mov     sp, offset stackend      ; readjust memory allocation
                mov     bx, offset buf_end       ;   to include 32k buffer
                mov     cl, 4
                shr     bx, cl
                mov     ah, 4Ah
                int     21h

                jnb     mem_ok
                mov     dx, offset no_mem        ; if carry, quit
                jmp     err_exit

mem_ok:         mov     cl, [ParamLen]           ; now check our parameters
                xor     ch, ch
                or      cx, cx
                jnz     got_parm

umsg:           mov     dx, offset usage         ; if none,  error

;*
;*  general exit-with-error code routine:
;*    print string pointed to by DX
;*    print CR/LF
;*    exit with return code 1
;*
err_exit:       mov     ah, 9
                int     21h
                mov     dx, offset crlf
                mov     ah, 9
                int     21h
                mov     ax, 4C01h
                int     21h

got_parm:       mov     di, 81h                  ; step one: get past spaces
                mov     al, ' '
                repe    scasb
                jz      umsg                     ; if nothing but,  error

                dec     di

                mov     al, [di]                 ; got our man
                cmp     al, 13
                je      umsg

                cmp     al, 'A'                  ; lower case it,  if
                jb      no_up                    ;   necessary
                cmp     al, 'Z'
                ja      no_up

                add     al, 'a' - 'A'

no_up:          cmp     al, 'm'                  ; our candidates
                je      got_val
                cmp     al, 'b'
                je      got_val
                cmp     al, 'a'
                je      got_val
                jmp     umsg                     ; none,  so error

got_val:        mov     val, al                  ; save param in 'val'

                mov     dx, offset fil           ; open DA.EXE
                mov     al, 0
                mov     ah, 3Dh
                int     21h
                jnc     got_fil

                mov     dx, offset no_fil
                jmp     err_exit

got_fil:        mov     handle, ax               ; save handle

                mov     ah, 3Fh                  ; read the whole thing
                mov     bx, handle
                mov     cx, bufsize
                mov     dx, offset buf

                int     21h
                jnc     read_ok

                mov     bx, handle               ; if carry,  close the file
                mov     ah, 3Eh                  ;   and exit
                int     21h

                jnc     close_ok1
                mov     dx, offset no_close_fil
                jmp     err_exit

close_ok1:      mov     dx, offset no_read
                jmp     err_exit

read_ok:        mov     left, ax                 ; save file size

                mov     bx, handle               ; close the file
                mov     ah, 3Eh
                int     21h

                jnc     close1_ok
                mov     dx, offset no_close_fil
                jmp     err_exit

close1_ok:      mov     di, offset buf           ; set up for search
                mov     cx, left

;*
;*  search buffer for string 'meth':
;*    base in DI,  byte count in CX
;*  find an m
;*    if no more m's,  exit
;*    else
;*      see if 'eth' follows it
;*
search_m:       mov     al, 'm'
                repne   scasb

                jcxz    no_m

                push    di
                push    cx
                mov     si, offset eth
                mov     cx, 3
                repe    cmpsb

                pop     cx

                je      got_eth

                pop     di
                jmp     search_m

got_eth:        pop     bx                       ; found 'meth'
                mov     al, val
                stosb                            ; append 'val'
                neg     al
                stosb                            ; append -'val'

                clc
                jmp     outs_m

no_m:           stc

outs_m:         jnc     write_it                 ; no carry, found 'meth'
                mov     dx, offset no_meth
                jmp     err_exit

write_it:       mov     dx, offset tmp           ; open 'DA.TMP'
                mov     ah, 3Ch
                mov     cx, 0
                int     21h

                jnc     got_tmp

                mov     dx, offset no_tmp
                jmp     err_exit

got_tmp:        mov     t_handle, ax             ; write 'DA.TMP'
                mov     ah, 40h
                mov     bx, t_handle
                mov     cx, left
                mov     dx, offset buf
                int     21h

                jnc     wrote_ok

                mov     bx, t_handle
                mov     ah, 3Eh
                int     21h

                jnc     close_ok2
                mov     dx, offset no_close_tmp
                jmp     err_exit

close_ok2:      mov     dx, offset no_write
                jmp     err_exit

wrote_ok:       mov     bx, t_handle             ; close 'DA.TMP'
                mov     ah, 3Eh
                int     21h
                jnc     close_ok3
                mov     dx, offset no_close_tmp
                jmp     err_exit

close_ok3:      mov     ah, 41h                  ; delete 'DA.EXE'
                mov     dx, offset fil
                int     21h

                mov     ah, 56h                  ; ren DA.TMP DA.EXE
                mov     dx, offset tmp
                mov     di, offset fil
                int     21h

                jnc     rename_ok
                mov     dx, offset no_rename
                jmp     err_exit

rename_ok:      mov     ax, 4C00h                ; and quit if no error
                int     21h
da_cfg  endp


;*
;*  equates for memory/stack allocation
;*
stackstart      label   byte                     ; bottom of stack
stackend        equ     stackstart + 100h        ; top of stack
buf             equ     stackend + 1             ; beginning of buffer
buf_end         equ     buf + bufsize            ; end of buffer

cseg    ends
                end     start

